package baka.dataflow;

import baka.gui.GraphicBlock;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 *
 * @author madda
 */
public abstract class DataBlock implements ActionListener {

    public String type;
    public String category;
    public String description = "";
    protected JPanel propertiesPanel;
    protected int maxInputCount = 5;
    protected int maxOutputCount = 5;
    protected int minInputCount = 0;
    protected int minOutputCount = 0;
    protected int inputCount = 1;
    protected int outputCount = 1;
    protected DataBus inputBus;
    protected DataBus outputBus;
    protected GraphicBlock graphicBlock;
    JComboBox inputComboBox;
    JComboBox outputComboBox;

    public void constructProperties() {
        this.propertiesPanel = new JPanel();
        this.propertiesPanel.setLayout(new GridLayout(0, 1));
        //this.propertiesPanel.setBorder(BorderFactory.createLineBorder(Color.red));
        JLabel caption = new JLabel("Block type: ");
        JTextField type = new JTextField(this.type);
        type.setEditable(false);

        this.propertiesPanel.add(this.createPropertiesPanelRow(caption, type));

        JLabel inputCaption = new JLabel("Number of Inputs");
        String[] inputs = new String[maxInputCount + 1];
        for (int i = minInputCount; i <= maxInputCount; i++) {
            inputs[i] = String.valueOf(i);
        }
        inputComboBox = new JComboBox(inputs);
        String selected = String.valueOf(this.inputCount);
        inputComboBox.setSelectedItem((Object) selected);
        inputComboBox.addActionListener(this);

        this.propertiesPanel.add(this.createPropertiesPanelRow(inputCaption, inputComboBox));

        JLabel outputCaption = new JLabel("Number of Outputs");
        String[] outputs = new String[maxOutputCount + 1];
        for (int i = minOutputCount; i <= maxOutputCount; i++) {
            outputs[i] = String.valueOf(i);
        }
        outputComboBox = new JComboBox(outputs);
        String selectedOutput = String.valueOf(this.outputCount);
        outputComboBox.setSelectedItem((Object) selectedOutput);
        outputComboBox.addActionListener(this);

        this.propertiesPanel.add(this.createPropertiesPanelRow(outputCaption, outputComboBox));

//        this.propertiesPanel.add(this.createPropertiesPanelRow(new JLabel("TEST"),new JTextField(this.type)));
//        this.propertiesPanel.add(this.createPropertiesPanelRow(new JLabel("TEST"),new JTextField(this.type)));
//        this.propertiesPanel.add(this.createPropertiesPanelRow(new JLabel("TEST"),new JTextField(this.type)));
//        this.propertiesPanel.add(this.createPropertiesPanelRow(new JLabel("TEST"),new JTextField(this.type)));
//        this.propertiesPanel.add(this.createPropertiesPanelRow(new JLabel("TEST"),new JTextField(this.type)));

        this.inputBus = new DataBus();
        this.outputBus = new DataBus();
    }

    protected JPanel createPropertiesPanelRow(Component caption, Component field) {
        JPanel ret = new JPanel(new BorderLayout());
        JPanel content = new JPanel(new GridLayout(1, 0));
        content.add(caption);
        content.add(field);
        ret.add(content, BorderLayout.NORTH);
        return ret;
    }

    public abstract void processInput();

    public abstract  void finalizeBlockWork();

    public Element getXML(Element e, Document doc) {
        Attr t = doc.createAttribute("type");
        t.setValue(this.type);
        e.setAttributeNode(t);


        Attr inpC = doc.createAttribute("inputCount");
        inpC.setValue(String.valueOf(this.inputCount));
        e.setAttributeNode(inpC);


        Attr outC = doc.createAttribute("outputCount");
        outC.setValue(String.valueOf(this.outputCount));
        e.setAttributeNode(outC);

        return e;
    }

    public void loadXML(Element e) {
        this.inputCount = Integer.parseInt(e.getAttribute("inputCount"));
        this.outputCount = Integer.parseInt(e.getAttribute("outputCount"));


    }

    public void setLoadedSettings() {
        String selected = String.valueOf(this.inputCount);
        inputComboBox.setSelectedItem((Object) selected);
        String selectedOutput = String.valueOf(this.outputCount);
        outputComboBox.setSelectedItem((Object) selectedOutput);
    }

    /**
     * Method executed on each block in conversion before conversion is executed.
     * Operations needed to to reset block state write here,
     * so conversion could be execute more than once.
     */
    public abstract void resetBlockState();

    public boolean execute() {
        System.out.print("execute na bloku: "+this.type);
        if (hasInputs()) {
            System.out.println(" ma vstupy spracuvavam");
            processInput();
            return true;
        } else {
            System.out.println(" nema vstupy nic sa nedeje");
            return false;
        }
    }

    public boolean hasInputs() {
        return inputBus.hasInputs();
    }

    public void sendOutputs() {
        outputBus.commit();
    }

    public Component getPropertiesPanel() {
        this.propertiesPanel.revalidate();
        JPanel ret = new JPanel(new BorderLayout());
        ret.add(this.propertiesPanel, BorderLayout.NORTH);
        JScrollPane scrollPane = new JScrollPane(ret);
        return scrollPane;
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() instanceof JComboBox) {
            JComboBox source = (JComboBox) e.getSource();
            if (source.equals(inputComboBox)) {
                //System.out.println("input event " + source.getSelectedItem());
                String tmp = (String) source.getSelectedItem();
                this.inputCount = Integer.parseInt(tmp);
                this.graphicBlock.repaint();
            } else if (source.equals(outputComboBox)) {
                //System.out.println("output event " + source.getSelectedItem());
                String tmp = (String) source.getSelectedItem();
                this.outputCount = Integer.parseInt(tmp);
                this.graphicBlock.repaint();
            }
        }
    }

    public int getInputCount() {
        return inputCount;
    }

    public int getOutputCount() {
        return outputCount;
    }

    public void setGraphicBlok(GraphicBlock gb) {
        this.graphicBlock = gb;
    }

    public void setBlockFile(String file){
    }
}
