/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package baka.gui;

import baka.dataflow.DataBlock;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.JPanel;
import javax.swing.event.MenuDragMouseEvent;

/**
 *
 * @author madda
 */
public class GraphicBlock extends JPanel implements MouseListener, MouseMotionListener {

    private int id, width, height;
    protected int inputDimension = 10;
    private DataBlock dataBlock;
    private GraphicCanvas parent;

    public GraphicBlock(int id, int width, int height, DataBlock db,GraphicCanvas parent) {
        this.id = id;
        this.width = width;
        this.height = height;
        this.dataBlock = db;
        this.addMouseListener(this);
        this.addMouseMotionListener(this);
        this.parent = parent;
        //this.setBorder(BorderFactory.createLineBorder(Color.BLACK));
    }

    @Override
    public void paintComponent(Graphics g) {
        Graphics2D g2 = (Graphics2D) g.create();
        paintInputs(g2);
        paintOutputs(g2);
        paintBlock(g2);
        g2.dispose();
    }

    private void paintBlock(Graphics2D g2) {
//        g2.setColor(new Color(145, 145, 145));
//        g2.fill(new Rectangle(5, 5, this.getWidth() - 10, this.getHeight() - 10));
//        g2.setColor(Color.BLACK);
//        g2.draw(new Rectangle(5, 5, this.getWidth() - 10, this.getHeight() - 10));
        BufferedImage bi = null;

        try {
            String specificImageLocation = "blocks/"+this.getDataBlock().category+"/"+ this.getDataBlock().type + ".gif";
            java.net.URL imgURL = getClass().getResource(specificImageLocation);
            try{
            bi = ImageIO.read(imgURL);
            }catch(IllegalArgumentException iae){

            }
        } catch (IOException ex) {
            Logger.getLogger(PalleteBlock.class.getName()).log(Level.SEVERE, null, ex);
        }
        if (bi == null) {
            String defaultImageLocation = "blocks/" + this.getDataBlock().category+ "/blok.gif";
            try {
                java.net.URL imgURL = getClass().getResource(defaultImageLocation);
                bi = ImageIO.read(imgURL);
            } catch (IOException ex) {
                Logger.getLogger(PalleteBlock.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        if (bi != null) {
            g2.drawImage(bi, 0, 5, null);
        }
        g2.setColor(Color.BLUE);
        g2.setFont(new Font(Font.SERIF, Font.BOLD, 13));
        g2.drawString(this.dataBlock.type,5,30);
    }

    private void paintInputs(Graphics2D g2) {
        Insets border = this.getInsets();
        int sirka = this.getWidth() - border.left - border.right;
        int inputsLengthArea = this.getInputCount() * this.inputDimension;
        int medzera = (sirka - inputsLengthArea) / (this.getInputCount() + 1);
        g2.setColor(Color.BLACK);
        for (int i = 0; i < getInputCount(); i++) {
            //g2.draw(new Rectangle(medzera * (i + 1) + inputDimension * i, 0, inputDimension, inputDimension));
            g2.fill(new Rectangle(medzera * (i + 1) + inputDimension * i, 0, inputDimension, inputDimension));
        }
    }

    private void paintOutputs(Graphics2D g2) {
        Insets border = this.getInsets();
        int sirka = this.getWidth() - border.left - border.right;
        int outputsLengthArea = this.getOutputCount() * this.inputDimension;
        int medzera = (sirka - outputsLengthArea) / (this.getOutputCount() + 1);

        int vyska = this.getHeight() - border.top - border.bottom;

        g2.setColor(Color.BLACK);
        for (int i = 0; i < this.getOutputCount(); i++) {
            //g2.draw(new Rectangle(medzera * (i + 1) + inputDimension * i, vyska - inputDimension, inputDimension, vyska));
            g2.fill(new Rectangle(medzera * (i + 1) + inputDimension * i, vyska - inputDimension, inputDimension, vyska));
        }
    }

    @Override
    public Dimension getPreferredSize() {
        return new Dimension(this.width, this.height);
    }

    public int isInputPoint(Point p) {
        Insets border = this.getInsets();
        int sirka = this.getWidth() - border.left - border.right;
        int inputsLengthArea = this.getInputCount() * this.inputDimension;
        int medzera = (sirka - inputsLengthArea) / (this.getInputCount() + 1);
        for (int i = 0; i < this.getInputCount(); i++) {
            Rectangle rect = new Rectangle(medzera * (i + 1) + inputDimension * i, 0, inputDimension, inputDimension);
            if (rect.contains(p)) {
                return i;
            }
        }
        return -1;
    }

    public int isOutputPoint(Point p) {
        Insets border = this.getInsets();
        int sirka = this.getWidth() - border.left - border.right;
        int outputsLengthArea = this.getOutputCount() * this.inputDimension;
        int medzera = (sirka - outputsLengthArea) / (this.getOutputCount() + 1);

        int vyska = this.getHeight() - border.top - border.bottom;

        for (int i = 0; i < this.getOutputCount(); i++) {
            Rectangle rect = new Rectangle(medzera * (i + 1) + inputDimension * i, vyska - inputDimension, inputDimension, vyska);
            if (rect.contains(p)) {
                return i;
            }
        }
        return -1;
    }

    public Point getDatovodCoords(DatovodPoint p) {
        Point ret = new Point();
        Insets border = this.getInsets();

        if (p.inputId != -1) {
            // input
            int sirka = this.getWidth() - border.left - border.right;
            int inputsLengthArea = this.getInputCount() * this.inputDimension;
            int medzera = (sirka - inputsLengthArea) / (this.getInputCount() + 1);
            ret.x = medzera * (p.inputId + 1) + inputDimension * p.inputId + inputDimension / 2;
            ret.y = 0;
        } else {
            int sirka = this.getWidth() - border.left - border.right;
            int outputsLengthArea = this.getOutputCount() * this.inputDimension;
            int medzera = (sirka - outputsLengthArea) / (this.getOutputCount() + 1);

            int vyska = this.getHeight() - border.top - border.bottom;
            //Rectangle rect = new Rectangle(medzera * (i + 1) + inputDimension * i, vyska - inputDimension, inputDimension, vyska);
            ret.x = medzera * (p.outputId + 1) + inputDimension * p.outputId + inputDimension / 2;
            ret.y = vyska;
        }

        return ret;
    }

    public int getId() {
        return id;
    }

    

    public int getInputCount() {
        return this.dataBlock.getInputCount();
    }

    public int getOutputCount() {
        return this.dataBlock.getOutputCount();
    }

    public DataBlock getDataBlock() {
        return dataBlock;
    }

    public void mouseClicked(MouseEvent e) {
        //System.out.println("GB klik at:" + e.getPoint());
        MouseEvent e1 = new MouseEvent(this, e.getID(), e.getWhen(), e.getModifiers(), e.getX() + this.getX(), e.getY() + this.getY(), e.getClickCount(), false);
        Component source = (Component) e.getSource();
        source.getParent().dispatchEvent(e1);
    }

    public void mousePressed(MouseEvent e) {
        MouseEvent e1 = new MouseEvent(this, e.getID(), e.getWhen(), e.getModifiers(), e.getX() + this.getX(), e.getY() + this.getY(), e.getClickCount(), false);
        Component source = (Component) e.getSource();
        source.getParent().dispatchEvent(e1);
    }

    public void mouseReleased(MouseEvent e) {
        MouseEvent e1 = new MouseEvent(this, e.getID(), e.getWhen(), e.getModifiers(), e.getX() + this.getX(), e.getY() + this.getY(), e.getClickCount(), false);
        Component source = (Component) e.getSource();
        source.getParent().dispatchEvent(e1);
    }

    public void mouseEntered(MouseEvent e) {
        MouseEvent e1 = new MouseEvent(this, e.getID(), e.getWhen(), e.getModifiers(), e.getX() + this.getX(), e.getY() + this.getY(), e.getClickCount(), false);
        Component source = (Component) e.getSource();
        source.getParent().dispatchEvent(e1);
    }

    public void mouseExited(MouseEvent e) {
        MouseEvent e1 = new MouseEvent(this, e.getID(), e.getWhen(), e.getModifiers(), e.getX() + this.getX(), e.getY() + this.getY(), e.getClickCount(), false);
        Component source = (Component) e.getSource();
        source.getParent().dispatchEvent(e1);
    }

    public void mouseDragged(MouseEvent e) {
        MouseEvent e1 = new MouseEvent(this, e.getID(), e.getWhen(), e.getModifiers(), e.getX() + this.getX(), e.getY() + this.getY(), e.getClickCount(), false);
        Component source = (Component) e.getSource();
        source.getParent().dispatchEvent(e1);
    }

    public void mouseMoved(MouseEvent e) {
        MouseEvent e1 = new MouseEvent(this, e.getID(), e.getWhen(), e.getModifiers(), e.getX() + this.getX(), e.getY() + this.getY(), e.getClickCount(), false);
        Component source = (Component) e.getSource();
        source.getParent().dispatchEvent(e1);
    }

    @Override
    public GraphicCanvas getParent() {
        return parent;
    }
    
}
