/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package baka.gui;

import baka.dataflow.DataBlock;
import baka.main.Config;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.JPanel;

/**
 *
 * @author madda
 */
public class GraphicCanvas extends JPanel implements KeyListener {

    VisualFrame parent;
    Vector<GraphicBlock> blocks;
    Vector<Datovod> connectors;
    DatovodPoint lastClickedInput;
    DatovodPoint lastClickedOutput;
    GraphicBlock lastInsertedBlock;
    GraphicBlock selectedBlock = null;
    static int gbCount = 0;
    Rectangle selectedArea = null;
    private int minimalWidth = 400;
    private int minimalHeight = 300;

    public GraphicCanvas(VisualFrame p) {
        super(null);
        this.parent = p;
        blocks = new Vector<GraphicBlock>();
        connectors = new Vector<Datovod>();

        lastClickedInput = new DatovodPoint();
        lastClickedOutput = new DatovodPoint();

        this.setBorder(BorderFactory.createLineBorder(Color.lightGray));
        GraphicCanvasMouseAdapter adapter = new GraphicCanvasMouseAdapter(this);
        this.addMouseListener(adapter);
        this.addMouseMotionListener(adapter);
        this.addKeyListener(this);
    }

    @Override
    public void paintComponent(Graphics g) {
        Graphics2D g2 = (Graphics2D) g.create();
        g2.setColor(Color.WHITE);
        g2.fill(new Rectangle(0, 0, this.getWidth() - 1, this.getHeight() - 1));
        if (selectedArea != null) {
            g2.setColor(Config.selectedArea);
            g2.fill(selectedArea);
            g2.setStroke(new BasicStroke(3));
            g2.setColor(Config.selectedAreaBorder);
            g2.draw(selectedArea);
        }
        paintConnectors(g2);
        g2.dispose();
    }

    public void dropNewBlock(String s) {
        Class[] classParm = null;
        Object[] objectParm = null;

        DataBlock dataBlock = null;
        try {
            String name = "baka.dataflow.blocks." + s;
            Class cl = Class.forName(name);
            java.lang.reflect.Constructor co = cl.getConstructor(classParm);
            dataBlock = (DataBlock) co.newInstance(objectParm);
        } catch (Exception e) {
            e.printStackTrace();
        }

        GraphicBlock gb = new GraphicBlock(this.gbCount, Config.BlockWidth, Config.BlockHeight, dataBlock, this);
        this.gbCount++;
        dataBlock.setGraphicBlok(gb);
        blocks.add(gb);
        this.add(gb);
        lastInsertedBlock = gb;
    }

    public void moveLastAddedBlock(Point p) {
        if (lastInsertedBlock != null) {
            Dimension dim = lastInsertedBlock.getPreferredSize();
            Insets insets = this.getInsets();
            lastInsertedBlock.setBounds(p.x + insets.left, p.y + insets.top, dim.width, dim.height);
        }
        this.repaint();
    }

    public void paintConnectors(Graphics2D g2) {
        for (int i = 0; i < connectors.size(); i++) {
            Datovod temp = connectors.get(i);
            paintDatovod(g2, temp.startPoint, temp.endPoint, temp.type);
        }
    }

    protected void paintDatovod(Graphics2D g2, DatovodPoint start, DatovodPoint end, int type) {
        Kabel temp = getDatovodKabel(start, end, type);
        temp.paintKabel(g2);
    }

    protected Kabel getDatovodKabel(DatovodPoint start, DatovodPoint end, int type) {
        // suradnice vramci blocku
        Point x1 = blocks.get(start.blockId).getDatovodCoords(start);
        Point x2 = blocks.get(end.blockId).getDatovodCoords(end);
        // suradnice blokov
        Point p1 = blocks.get(start.blockId).getLocation();
        Point p2 = blocks.get(end.blockId).getLocation();

        p1.x += x1.x;
        p1.y += x1.y;
        p2.x += x2.x;
        p2.y += x2.y;

        Kabel temp = new Kabel(p1, p2, type);
        return temp;
    }

    public void addBlock(GraphicBlock b) {
        this.blocks.add(b);
    }

    public int getClickedBlock(Point p) {
        for (int i = 0; i < blocks.size(); i++) {
            GraphicBlock gb = blocks.get(i);
            Rectangle rect = gb.getBounds();
            if (rect.contains(p)) {
                return i;
            }
        }
        return -1;
    }

    public void clearCanvas() {
        this.removeAll();
        this.blocks = new Vector<GraphicBlock>();
        this.connectors = new Vector<Datovod>();
        this.gbCount = 0;
        this.repaint();
    }

    public void load(Vector<GraphicBlock> gblocks, Vector<Datovod> datovods) {
        this.gbCount = gblocks.size();
        this.blocks = gblocks;
        this.connectors = datovods;
        int maxX = 0, maxY = 0;
        for (int i = 0; i < blocks.size(); i++) {
            GraphicBlock temp = blocks.elementAt(i);
            if (temp.getX() > maxX) {
                maxX = temp.getX();
            }
            if (temp.getY() > maxY) {
                maxY = temp.getY();
            }
            this.add(temp);
        }
        maxX += 150;
        maxY += 150;
        if (maxX < minimalWidth) {
            maxX = minimalWidth;
        }
        if (maxY < minimalHeight) {
            maxY = minimalHeight;
        }
        this.setPreferredSize(new Dimension(maxX, maxY));
        this.revalidate();
        this.repaint();
    }

    public boolean hasConnector(Datovod d) {
        for (int i = 0; i < connectors.size(); i++) {
            Datovod temp = connectors.get(i);
            if (temp.equals(d)) {
                return true;
            }
        }
        return false;
    }

    public boolean availableConnector(Datovod d) {
        for (int i = 0; i < connectors.size(); i++) {
            Datovod temp = connectors.get(i);
            if (temp.getStartPoint().equals(d.getStartPoint())) {
                return false;
            }
            if (temp.getEndPoint().equals(d.getEndPoint())) {
                return false;
            }
        }
        return true;
    }

    public void removeBlockConnectors(int blockId) {
        Vector<Datovod> toDelete = new Vector<Datovod>();
        for (int i = 0; i < connectors.size(); i++) {
            Datovod temp = connectors.get(i);
            System.out.println("overujem datovod:" + temp);
            if (temp.getStartPoint().getBlockId() == blockId) {
                System.out.println("start je zly,mazem");
                toDelete.add(temp);
            } else if (temp.getEndPoint().getBlockId() == blockId) {
                toDelete.add(temp);
                System.out.println("koniec je zly,mazem");
            }
        }

        for (int i = 0; i < toDelete.size(); i++) {
            connectors.remove(toDelete.get(i));
        }
    }

    public void keyTyped(KeyEvent e) {
    }

    public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == 127) {
            //System.out.println("remove selected code");
            int id = this.selectedBlock.getId();
            removeBlockConnectors(id);
            this.remove(this.selectedBlock);
            this.blocks.remove(this.selectedBlock);
            this.selectedBlock = null;
            System.out.println("Zmazal som block:" + id);
            this.repaint();
        }
    }

    public void keyReleased(KeyEvent e) {
    }

    public class Kabel {

        private Point start;
        private Point end;
        private int type;
        Vector<Linka> lines;

        public Kabel(Point start, Point end, int type) {
            this.start = start;
            this.end = end;
            this.type = type;
            this.lines = new Vector<Linka>();

            if (Math.abs(start.x - end.x) > 5) {
                // do p1 ulozim Point s mensou Y suradnicou
                if (start.y > end.y) {
                    Point x1 = start;
                    start = end;
                    end = x1;
                }
                // musim kreslit viac ako jednu ciaru;
                int stredY = Math.abs(start.y - end.y) / 2;
                Point temp = new Point();
                temp.x = start.x;
                temp.y = start.y + stredY;
                this.lines.add(new Linka(start, temp));
                Point temp2 = new Point(end.x, start.y + stredY);
                this.lines.add(new Linka(temp, temp2));
                this.lines.add(new Linka(temp2, end));
            } else {
                this.lines.add(new Linka(start, end));
            }
        }

        public void paintKabel(Graphics2D g2) {
            if (this.type == 1) {
                // limited
                g2.setColor(new Color(230, 50, 50));
            } else if (this.type == 2) {
                g2.setColor(new Color(230, 230, 50));
            } else {
                // unlimited
                g2.setColor(new Color(50, 230, 50));
            }

            g2.setStroke(new BasicStroke(6));
            for (int i = 0; i < this.lines.size(); i++) {
                this.lines.get(i).paintLine(g2);
            }
        }

        public class Linka {

            private Point start;
            private Point end;

            public Linka(Point start, Point end) {
                this.start = start;
                this.end = end;
            }

            public void paintLine(Graphics2D g2) {
                g2.drawLine(start.x, start.y, end.x, end.y);
            }
        }
    }
}
